/*
*	This file is part of the EMFexplorer projet.
*	Copyright (C) 2004 Smith Charles.
*
*	This library is free software; you can redistribute it and/or
*	modify it under the terms of the GNU Lesser General Public
*	License as published by the Free Software Foundation; either
*	version 2.1 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public
*   License along with this library; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
*
*	Extension: for commercial use, apply the Equity Public License, which
*	adds to the normal terms of the GLPL a condition of donation to the author.
*   If you are interested in support for this source code,
*   contact Smith Charles <smith.charles@free.fr> for more information.
*/
#ifndef _SCGENDEFS_H_
#define _SCGENDEFS_H_

// This macro is used to reduce size requirements of some classes
#ifndef SMC_ALWAYS_VTABLE
	#ifndef SMC_NOVTABLE
		#if _MSC_VER >= 1100 && !defined(_DEBUG)
			#define SMC_NOVTABLE __declspec(novtable)
		#else
			#define SMC_NOVTABLE
		#endif
	#endif
#else
			#define SMC_NOVTABLE
#endif

// This macro is used to declare interface classes
#define SMC_INTERFACE SMC_NOVTABLE

// Delete an object and NULL out the pointer so that an error is generated
// if it is reused without reallocation
#define SMC_SAFEDELETE(p)	\
	if (p)					\
	{						\
		delete p;			\
		p = NULL;			\
	}

#define SMC_SAFEARRAY_DELETE(p)	\
	if (p)						\
	{							\
		delete [] p;			\
		p = NULL;				\
	}

// Delete with test (typically to call in destructors)
#define SMC_DELETE(p)		\
	if (p)					\
		delete p

#define SMC_ARRAY_DELETE(p)		\
	if (p)						\
		delete [] p

// Swap anything having an assignment or copy operator
template <class T>
inline void SCSwap(T& rA, T& rB)
{
	T tTmp = rA; rA = rB; rB = tTmp;
}

// Swap ints (presumably)
#define SMC_ISWAP(x, y) SCSwap(x, y)

// Constrain l-value to range
#define SC_LIMIT_VALUE(v_lvalue, vMin, vMax)	\
	if (v_lvalue>(vMax))						\
		v_lvalue = (vMax);						\
	else										\
	if (v_lvalue<(vMin))						\
		v_lvalue = (vMin)

#endif //_SCGENDEFS_H_
//  ------------------------------------------------------------
